@@ -1,14 +1,16 @@
 #' Merge \code{inla} objects for partition models
 #'
 #' @description The function takes local models fitted for each subregion of the whole spatial domain and unifies them into a single \code{inla} object.
-#' This function is valid for both disjoint and \emph{k}-order neighbourhood models.
+#' This function is valid for both Disjoint and \emph{k}-order neighbourhood models.
 #'
-#' @details If the disjoint model is fitted (\code{k=0} argument), the log-risk surface is just the union of the posterior estimates of each submodel.
-#' However, to obtain a single estimate of the overall log-risk \eqn{\alpha}, we extract samples from the joint posterior distribution of the linear predictors using the \code{inla.posterior.sample} function of R-INLA.
+#' @details If the Disjoint model is fitted (\code{k=0} argument), the log-risk surface is just the union of the posterior estimates of each submodel.
+#' However, a single estimate of the overall log-risk \eqn{\alpha} can be computed by extracting samples from the joint posterior distribution of the linear predictors using the \code{inla.posterior.sample} function of R-INLA.
 #' After joining the \eqn{S} samples from each submodel, we define \deqn{\alpha^s=\frac{1}{nT}\sum_{i=1}^n\sum_{t=1}^T \log{r_{it}}, \quad \mbox{for} \quad s=1,\ldots,S} and then compute the kernel density estimate of \eqn{\alpha}.
 #' \cr \cr
 #' If the \emph{k}-order neighbourhood model is fitted (\code{k>0} argument), note that the final risk surface \eqn{{\bf r}=(r_1,\ldots,r_{nT})^{'}} is no longer the union of the posterior estimates obtained from each submodel.
-#' To obtain a unique posterior distribution of \eqn{r_{it}} for each areal unit \eqn{i} and time point \eqn{t}, a mixture distribution of the estimated posterior probability density functions is computed using the conditional predictive ordinates (CPO) to compute the mixture weights.
+#' Since multiple log-risk estimates can be obtained for some areal-time units from the different local submodel, their posterior estimates must be properly combined to obtain a single posterior distribution for each \eqn{r_{it}}.
+#' Two different merging strategies could be considered. If the \code{merge.strategy="mixture"} argument is specified, mixture distributions of the estimated posterior probability density functions with weights proportional to the conditional predictive ordinates (CPO) are computed.
+#' If the \code{merge.strategy="original"} argument is specified (default option), the posterior marginal estimate ot the areal-unit corresponding to the original submodel is selected.
 #' \cr \cr
 #' See \insertCite{orozco2020;textual}{bigDM} and \insertCite{orozco2022;textual}{bigDM} for more details.
 #'
@@ -21,8 +23,9 @@
 #' @param seed numeric; control the RNG of \code{inla.qsample} (see \code{help(inla.qsample)} for further information). Defaults to \code{NULL}.
 #' @param n.sample numeric; number of samples to generate from the posterior marginal distribution of the risks. Default to 1000.
 #' @param compute.fixed logical value (default \code{FALSE}); if \code{TRUE} then the overall log-risk \eqn{\alpha} is computed.
-#' Only works if \code{k=0} argument (\emph{disjoint model}) is specified.
+#' Only works if \code{k=0} argument (\emph{Disjoint model}) is specified. CAUTION: This method might be very time consuming.
 #' @param compute.DIC logical value; if \code{TRUE} (default) then approximate values of the Deviance Information Criterion (DIC) and Watanabe-Akaike Information Criterion (WAIC) are computed.
+#' @param merge.strategy one of either \code{"mixture"} or \code{"original"} (default), which specifies the merging strategy to compute posterior marginal estimates of relative risks.
 #'
 #' @return This function returns an object of class \code{inla} containing the following elements:
 #' \item{\code{summary.fixed}}{If \code{compute.fixed=TRUE} a data.frame containing the mean, standard deviation, quantiles and mode of the model's intercept.}
@@ -40,6 +43,7 @@
 #' \item{\code{cpu.used}}{The sum of cpu times used by the \code{inla} function for each submodel (\code{Pre}, \code{Running} and \code{Post}), and the cpu time of the merging process \code{Merging}.}
 #'
 #' @import parallel
+#' @importFrom methods is
 #' @importFrom stats runif density sd quantile dpois var
 #' @importFrom rlist list.flatten
 #'
@@ -47,14 +51,15 @@
 #' ## See the vignettes accompanying this package for an example of its use.
 #'
 #' @export
-mergeINLA <- function(inla.models=list(), k=NULL, ID.area="Area", ID.year=NULL, O="O", E="E", seed=NULL, n.sample=1000, compute.fixed=FALSE, compute.DIC=TRUE){
+mergeINLA <- function(inla.models=list(), k=NULL, ID.area="Area", ID.year=NULL, O="O", E="E", seed=NULL, n.sample=1000, compute.fixed=FALSE, compute.DIC=TRUE,
+                      merge.strategy="original"){
 
   if(requireNamespace("INLA", quietly=TRUE)){
 
           D <- length(inla.models)
 
           ## Check for errors ##
-          if(class(inla.models)!="list" | D==0)
+          if(!is(inla.models,"list") | D==0)
                   stop("the 'inla.models' argument must be a non-empty list")
           if(unlist(unique(lapply(inla.models, function(x) class(x))))!="inla")
                   stop("the 'inla.models' argument must contain 'inla' class objects")
@@ -66,6 +71,8 @@ mergeINLA <- function(inla.models=list(), k=NULL, ID.area="Area", ID.year=NULL,
                   stop("the 'O' argument is missing")
           if(is.null(E))
                   stop("the 'E' argument is missing")
+          if(!(merge.strategy %in% c("mixture","original")))
+                  stop("invalid 'merge.strategy' argument")
 
           tt <- system.time({
 
@@ -91,9 +98,19 @@ mergeINLA <- function(inla.models=list(), k=NULL, ID.area="Area", ID.year=NULL,
                   if(is.null(ID.year)){
                           ID.list <- lapply(inla.models, function(x) x$.args$data[,ID.area])
                           ID <- sort(as.character(unique(unlist(ID.list))))
+
+                          ID.group <- do.call(rbind,lapply(inla.models, function(x) x$.args$data[,c(ID.area,"ID.group")]))
+                          ID.group <- ID.group[!duplicated(ID.group[,ID.area]),]
+                          ID.group <- ID.group[order(ID.group[,ID.area]),]
+                          rownames(ID.group) <- ID
                   }else{
                           ID.list <- lapply(inla.models, function(x) paste(x$.args$data[,ID.year],x$.args$data[,ID.area],sep="."))
                           ID <- sort(as.character(unique(unlist(ID.list))))
+
+                          ID.group <- do.call(rbind,lapply(inla.models, function(x) x$.args$data[,c(ID.area,ID.year,"ID.group")]))
+                          ID.group <- ID.group[!duplicated(ID.group[,c(ID.area,ID.year)]),]
+                          ID.group <- ID.group[order(ID.group[,ID.year],ID.group[,ID.area]),]
+                          rownames(ID.group) <- ID
                   }
 
                   ## Fixed effects ##
@@ -275,7 +292,7 @@ mergeINLA <- function(inla.models=list(), k=NULL, ID.area="Area", ID.year=NULL,
 
                           suppressWarnings({
                                   cl <- makeCluster(detectCores())
-                                  clusterExport(cl, varlist=c("models.summary.fitted.values","models.marginals.fitted.values","models.cpo","ID","ID.list"), envir=environment())
+                                  clusterExport(cl, varlist=c("models.summary.fitted.values","models.marginals.fitted.values","models.cpo","ID","ID.list","merge.strategy","ID.group"), envir=environment())
                                   clusterEvalQ(cl,{
                                           INLA::inla.dmarginal
                                           INLA::inla.qmarginal
@@ -525,34 +542,44 @@ computeFittedValues <- function(q){
     marginals.fitted.values <- models.marginals.fitted.values[[i]][pos[[i]]]
     names(marginals.fitted.values) <- q
   }else{
-    post.mean <- mapply(function(x,y){x$mean[y]}, x=models.summary.fitted.values[i], y=pos[i])
-    post.sd <- mapply(function(x,y){x$sd[y]}, x=models.summary.fitted.values[i], y=pos[i])
-    post.cdf <- mapply(function(x,y){x$'1 cdf'[y]}, x=models.summary.fitted.values[i], y=pos[i])
-    marginals <- mapply(function(x,y){x[[y]]}, x=models.marginals.fitted.values[i], y=pos[i], SIMPLIFY=FALSE)
+    if(merge.strategy=="original"){
+            aux <- ID.group[which(rownames(ID.group)==q),"ID.group"]
 
-    cpo <- mapply(function(x,y){x$cpo[y]}, x=models.cpo[i], y=pos[i])
-    w <- cpo/sum(cpo)
+            summary.fitted.values <- models.summary.fitted.values[[aux]][pos[[aux]],1:7]
+            rownames(summary.fitted.values) <- q
 
-    xx <- sort(unlist(lapply(marginals, function(x) x[,"x"])))
-    at <- round(seq(1,length(xx),length.out=75))
-    marginals.fitted.values <- matrix(0, nrow=0, ncol=2, dimnames=list(NULL, c("x","y")))
-    for(j in xx[at]){
-      aux <- unlist(lapply(marginals, function(x) INLA::inla.dmarginal(j,x)))
-      marginals.fitted.values <- rbind(marginals.fitted.values,c(j,sum(aux*w)))
+            marginals.fitted.values <- models.marginals.fitted.values[[aux]][pos[[aux]]]
+            names(marginals.fitted.values) <- q
+    }else{
+            post.mean <- mapply(function(x,y){x$mean[y]}, x=models.summary.fitted.values[i], y=pos[i])
+            post.sd <- mapply(function(x,y){x$sd[y]}, x=models.summary.fitted.values[i], y=pos[i])
+            post.cdf <- mapply(function(x,y){x$'1 cdf'[y]}, x=models.summary.fitted.values[i], y=pos[i])
+            marginals <- mapply(function(x,y){x[[y]]}, x=models.marginals.fitted.values[i], y=pos[i], SIMPLIFY=FALSE)
+
+            cpo <- mapply(function(x,y){x$cpo[y]}, x=models.cpo[i], y=pos[i])
+            w <- cpo/sum(cpo)
+
+            xx <- sort(unlist(lapply(marginals, function(x) x[,"x"])))
+            at <- round(seq(1,length(xx),length.out=75))
+            marginals.fitted.values <- matrix(0, nrow=0, ncol=2, dimnames=list(NULL, c("x","y")))
+            for(j in xx[at]){
+                    aux <- unlist(lapply(marginals, function(x) INLA::inla.dmarginal(j,x)))
+                    marginals.fitted.values <- rbind(marginals.fitted.values,c(j,sum(aux*w)))
+            }
+
+            summary.fitted.values <- data.frame(sum(post.mean*w),
+                                                sqrt(sum((post.sd^2+post.mean^2-sum(post.mean*w)^2)*w)),
+                                                INLA::inla.qmarginal(0.025,marginals.fitted.values),
+                                                INLA::inla.qmarginal(0.5,marginals.fitted.values),
+                                                INLA::inla.qmarginal(0.975,marginals.fitted.values),
+                                                INLA::inla.mmarginal(marginals.fitted.values),
+                                                sum(post.cdf*w))
+            colnames(summary.fitted.values) <- c("mean","sd","0.025quant","0.5quant","0.975quant","mode","1 cdf")
+            rownames(summary.fitted.values) <- q
+
+            marginals.fitted.values <- list(marginals.fitted.values)
+            names(marginals.fitted.values) <- q
     }
-
-    summary.fitted.values <- data.frame(sum(post.mean*w),
-                                        sqrt(sum((post.sd^2+post.mean^2-sum(post.mean*w)^2)*w)),
-                                        INLA::inla.qmarginal(0.025,marginals.fitted.values),
-                                        INLA::inla.qmarginal(0.5,marginals.fitted.values),
-                                        INLA::inla.qmarginal(0.975,marginals.fitted.values),
-                                        INLA::inla.mmarginal(marginals.fitted.values),
-                                        sum(post.cdf*w))
-    colnames(summary.fitted.values) <- c("mean","sd","0.025quant","0.5quant","0.975quant","mode","1 cdf")
-    rownames(summary.fitted.values) <- q
-
-    marginals.fitted.values <- list(marginals.fitted.values)
-    names(marginals.fitted.values) <- q
   }
   return(list(summary.fitted.values,marginals.fitted.values))
 }
@@ -573,4 +600,6 @@ utils::globalVariables(c("ID.list",
                          "inla.qmarginal",
                          "inla.mmarginal",
                          "inla.make.lincombs",
-                         "inla"))
+                         "inla",
+                         "merge.strategy",
+                         "ID.group"))
